#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from . import mpfun
from models import *
from sqlalchemy.orm import Session
from database import get_local_db

# 灾害信息员信息表

# 加密和HMAC签名
def sign_row(db: Session, row: GovdataDisasterInfoOfficer) -> None:
    if row.sign != '':
        return
    
    name = mpfun.base64_data(row.name) # 姓名
    gender = mpfun.base64_data(row.gender) # 性别
    work_unit = mpfun.base64_data(row.work_unit) # 工作单位
    cellphone = mpfun.enc_data(row.cellphone) # 手机号码
    location = mpfun.base64_data(row.location) # 所处位置
    area = mpfun.base64_data(row.area) # 区县
    longitude = str(row.longitude) # 经度
    latitude = str(row.latitude) # 纬度
    del_flag = row.del_flag # 是否已删除

    sign_data = ",".join([name, gender, work_unit, cellphone, location, area, longitude, latitude, del_flag])
    sign_hmac = mpfun.sign_data(sign_data)
    # print('sign_data:', sign_data)
    # print('sign_hmac:', sign_hmac)

    row.cellphone = cellphone
    row.sign = sign_hmac
    
    db.commit()

# 比较字段合并字符串是否和MAC值匹配上,调用密码服务器[验证HMAC]接口
def sign_valid_row(row: GovdataDisasterInfoOfficer) -> bool:
    return True

    if row.sign == '':
        return True

    # 关键字段合并字符串
    sign_data = get_sign_str(row)
    # print('sys_user sign_data:', sign_data)
    
    # 原HMACSM3数值
    sign_hmac = row.sign
    # print('sign_hmac:', sign_hmac)

    return mpfun.hmac_verify(sign_data, sign_hmac)

# 生成待签名的字符串
def get_sign_str(row: GovdataDisasterInfoOfficer) -> str:
    name = mpfun.base64_data(row.name) # 姓名
    gender = mpfun.base64_data(row.gender) # 性别
    work_unit = mpfun.base64_data(row.work_unit) # 工作单位
    cellphone = (row.cellphone) # 手机号码
    location = mpfun.base64_data(row.location) # 所处位置
    area = mpfun.base64_data(row.area) # 区县
    longitude = str(row.longitude) # 经度
    latitude = str(row.latitude) # 纬度
    del_flag = row.del_flag # 是否已删除

    # 关键字段合并字符串
    sign_data = ",".join([name, gender, work_unit, cellphone, location, area, longitude, latitude, del_flag])
    return sign_data

# 生成HAMC签名值
def get_sign_hmac(row: GovdataDisasterInfoOfficer) -> str:
    sign_data = get_sign_str(row)
    return mpfun.sign_data(sign_data) 

# 对所有数据进行签名
def sign_table():
    print('sign_govdata_disaster_info_officer table =====>>>')
    with get_local_db() as db:
        rows = db.query(GovdataDisasterInfoOfficer).filter(GovdataDisasterInfoOfficer.sign == '').all()
        for row in rows:
            sign_row(db, row)